
# q-expansion of newform 441.6.a.g, downloaded from the LMFDB on 18 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 441
weight = 6
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[2], [0], [-11], [0], [-269], [-308], [-1896], [-164], [3264], [-2417], [2841], [-11328], [16856], [-7894], [-21102], [29691], [8163], [15166], [-32078], [38274], [34866], [13529], [68103], [114922], [154959], [107570], [8936], [-193667], [205110], [-46664], [-304365], [13303], [398262], [230286], [97134], [-29047], [-576500], [-265232], [363790], [164846], [-30628], [-651392], [757360], [-160339], [61738], [370908], [217450], [589771], [387045], [-232732], [-42096], [313416], [857807], [-454517], [-878182], [-1960934], [1053795], [-105059], [-427592], [-638878], [-2451422], [-1716169], [1808968], [-1521456], [-1348399], [49695], [1587836], [214825], [-2588596], [-24878], [1736008], [-862426], [3115419], [-1796944], [3454656], [-2155038], [462774], [-4066212], [5068632], [2877335], [3413424], [-1307374], [-1935468], [516670], [2915301], [1783791], [-4001580], [-1167665], [-3613582], [-1801112], [2369748], [-518146], [2826127], [-9347473], [8171854], [7377132], [326315], [2167026], [-723404], [5999642], [7015696], [8918725], [13348231], [-1102148], [1893482], [2829511], [10679893], [14699688], [-8491626], [8629467], [10580735], [3847840], [15100062], [9931018], [-9252241], [-5004282], [12613706], [12538310], [8660661], [-7941768], [-2114162], [-442307], [-10756071], [11488593], [10138832], [19683879], [-20171706], [-4157346], [-15443323], [-6204138], [21898446], [2754834], [12912079], [-26431522], [-12221388], [6106541], [3022202], [-20828522], [-23232839], [-10866768], [-22263178], [12388091], [1694808], [378495], [-10428748], [-30408224], [28031486], [-18802958], [7863230], [11285766], [13415008], [-31854690], [-30592168], [4637720], [604376], [24405918], [36709496], [22908854], [-5996109], [11651585], [1109264], [-10574314], [6326665], [-39239463], [1550742], [48748428], [-1925515], [54256390], [14158111], [-20562679], [-29542635], [-38633658], [-723588], [-25319179], [29030399], [19142900], [-2709698], [57314905], [61190640], [29055058], [-12368407], [59704029], [-64133242], [40891322], [40077594], [-45715627], [20193358], [46258414], [26327299], [-21666140], [-27706703], [19695933], [-37070140], [-79869066], [-49755233], [28097574], [76917830], [-903114], [-36276702], [12323538], [75100925], [88677848], [-42960522], [62435050], [-14714560], [65456903], [-87932736], [-95558811], [-64177920], [-45908822], [91875138], [34302849], [42256825], [73493605], [-111094444], [104166514], [14525843], [-73624592], [-23221910], [-6817134], [-132439624], [-22565816], [101539286], [-36923539], [-37433263], [27876704], [-27564674], [-56866036], [144928277], [117032652], [20782114], [76539112], [-55404823], [-8724614], [-104565510], [-93192756], [-144842477], [157090985], [69177036], [-71350552], [78326700], [-125684688], [5673891], [-11897342], [-172272156], [13868956], [-152265071], [112206956], [-149454848], [187069776], [149254730], [-131567202], [15000934], [125379058], [60650227], [-30483661], [153471482], [-3892596], [-103371093], [83743853], [-59547184], [166401191], [-18463306], [-130740316], [-16093426], [198146750], [65540025], [-148068452], [-57161732], [-92440127], [-38031920], [-75012718], [-191908571], [15189767], [-7228907], [-131676352], [-119471315], [-103273776], [98890678], [77824736], [98980610], [155525709], [56403116], [-187066628], [-209035690], [9405216], [192503671], [65240145], [190176335], [296787036], [106940950], [-143025875], [-182030364], [-134432172], [126514684], [-153781823], [168461248], [330541190], [172795556], [-52310146], [106973094], [81092097], [-289248593], [-179510417], [32804406], [141970866], [282247095], [6301638], [227093638], [160587334], [307897614], [-19710862], [105914244], [-66493872], [170537288], [-201042682], [182356587], [85083912], [61215301], [-94482936], [2837120], [172106002], [299825799], [323874896], [-57088776], [66083371], [88083154], [46366646], [-426227590], [-314224563], [135453106], [114675369], [143968496], [-458184], [205908538], [68296900], [-211782100], [140317698], [-27327223], [63432331], [75675190], [-111948748], [79815519], [-313919093], [401900207], [180893305], [229989430], [427613926], [262689714], [219322967], [-3929684], [-16559253], [-348243254], [-144256113], [-83156138], [-218369242], [-235998270], [6913971], [-302289244], [-403849602], [-40949550], [-532772552], [52366122], [293562572], [461548856], [285065779], [-39120214], [113164936], [-233329143], [-121130355], [595124688], [291054318], [297630975], [-447768784], [-355988096], [499800904], [123125308], [651505259], [-248777778], [457016910], [117103190], [-68711190], [221214688], [421424084], [-144373362], [200089226], [85876640], [-673547876], [-378207916], [-302811012], [-570049415], [-103881306], [-649291065], [-498209134], [-121110778], [-6405977], [-193675049], [-307643658], [650350080], [-336123735], [368220332], [-184821376], [-16311500], [-143233458], [-160497357], [542023675], [134848374], [-63635098], [-198987538], [547112415], [-691248673], [495450797], [-809194087], [718239310], [-185829222], [-91289517], [172441908], [-235006360], [-875314608], [617150922], [314557989], [382507318], [93634658], [26093711], [255188234], [-873249054], [-259983323], [202090488], [522132666], [963729684]]
